package cz.muni.fi.pa165.icehockeymanager.rest.demo;

import cz.muni.fi.pa165.icehockeymanager.dto.*;
import cz.muni.fi.pa165.icehockeymanager.facades.LeagueManagerFacade;
import cz.muni.fi.pa165.icehockeymanager.facades.TeamManagerFacade;
import cz.muni.fi.pa165.icehockeymanager.facades.UserAuthFacade;
import cz.muni.fi.pa165.icehockeymanager.facades.UserFacade;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
public class DataLoader implements ApplicationRunner {

    private final UserAuthFacade userAuth;
    private final LeagueManagerFacade leagueManager;
    private final TeamManagerFacade teamManagerFacade;
    private final UserFacade userFacade;

    public DataLoader(UserAuthFacade userAuth, LeagueManagerFacade leagueManager, TeamManagerFacade teamManagerFacade, UserFacade userFacade) {
        this.userAuth = userAuth;
        this.leagueManager = leagueManager;
        this.teamManagerFacade = teamManagerFacade;
        this.userFacade = userFacade;
    }

    private static TeamCreateDto createTeam(String name) {
        var team = new TeamCreateDto();
        team.setName(name);
        return team;
    }

    private static PlayerCreateDto createPlayer(String name, long teamId) {
        var player = new PlayerCreateDto();
        player.setName(name);
        player.setTeamId(teamId);
        return player;
    }

    private static PlayerTransferDto transferPlayer(long playerId, long teamId) {
        var transfer = new PlayerTransferDto();
        transfer.setPlayerId(playerId);
        transfer.setTeamId(teamId);
        return transfer;
    }

    @Override
    public void run(ApplicationArguments args) {
        leagueManager.createNewTeam(createTeam("Toronto Maple Leafs"));
        leagueManager.createNewTeam(createTeam("HC Slovan Bratislava"));
        List<TeamDto> teams = new ArrayList<>(userFacade.getTeamsInLeague());

     /*   teamManagerFacade.recruitNewPlayer(createPlayer("Marian Hossa", teams.get(0).getId()));
        teamManagerFacade.recruitNewPlayer(createPlayer("Marcel Hossa", teams.get(1).getId()));
        teamManagerFacade.recruitNewPlayer(createPlayer("Zdeno Chára", teams.get(0).getId()));
        teamManagerFacade.recruitNewPlayer(createPlayer("Miroslav Šatan", teams.get(1).getId()));*/

        userAuth.createTeamManager("team_manager", "1234", teams.get(0).getId().intValue());
       // userAuth.createTeamManager("team_manager_slovan", "0000", teams.get(1).getId().intValue());
    }
}
