import {Injectable} from '@angular/core';
import {HttpClient} from '@angular/common/http';

import {Observable, of} from 'rxjs';
import {AccessEnum} from '../models/access-enum';
import {AssociatedTeam} from '../models/associated-team';
import {CreateTeam} from '../models/create-team';

import {Team} from '../models/team'

@Injectable({
  providedIn: 'root'
})
export class TeamService {

  teams: Team[] = [
    {id: 1, name: 'Toronto Maple Leafs'},
    {id: 2, name: 'Boston Bruins'}
  ];

  getAllUrl: string = 'http://localhost:8080/pa165/api/public/team/all';
  createUrl: string = 'http://localhost:8080/pa165/api/manage/league/team/create';

  constructor(private http: HttpClient) {
  }

  getTeams(): Observable<Team[]> {
    return this.http.get<Team[]>(this.getAllUrl);
  }

  createTeam(team: CreateTeam): void {
    this.http.post<Team>(this.createUrl, team)
      .subscribe();
  }

  getAssociatedTeam(): Observable<AssociatedTeam> {
    return of({
      stats: AccessEnum.OK,
      team: this.teams[0]
    });
  }
}
